#include "mbed.h"
#include "USBSerial.h"
#include "BLEPeripheral.h"

//Serial serial = Serial(USBTX, USBRX);
USBSerial serial = USBSerial();

DigitalOut led = DigitalOut(P1_5);

// The SPI construct, REQN and RDYN IO construct should be modified manually
// It depend on the board you are using and the REQN&RDYN configuration on BLE Shield
SPI spi = SPI(P1_22, P1_21, P1_20);
DigitalInOut bleREQ = DigitalInOut(P1_27);
DigitalInOut bleRDY = DigitalInOut(P1_26);

const char CR = 13;
unsigned char txbuf[16] = {0};
unsigned char txlen = 0;

/*----- BLE Utility -------------------------------------------------------------------------*/
// create peripheral instance, see pinouts above
BLEPeripheral blePeripheral = BLEPeripheral(&bleREQ, &bleRDY, NULL);
// create service
BLEService uartService = BLEService("713d0000503e4c75ba943148f18d941e");
// create characteristic
BLECharacteristic txCharacteristic = BLECharacteristic("713d0002503e4c75ba943148f18d941e", BLENotify, 20);
BLECharacteristic rxCharacteristic = BLECharacteristic("713d0003503e4c75ba943148f18d941e", BLEWriteWithoutResponse, 20);
/*--------------------------------------------------------------------------------------------*/

void setup()
{
    led = 0;
    for (int i = 0; i < 6; i++) {
        led = !led;
        wait(0.2);
    }

    //serial.baud(115200);
    serial.printf("Serial begin!\r\n");

    /*----- BLE Utility ---------------------------------------------*/
    // set advertised local name and service UUID
    blePeripheral.setLocalName("BLE Shield");
    blePeripheral.setAdvertisedServiceUuid(uartService.uuid());
    // add service and characteristic
    blePeripheral.addAttribute(uartService);
    blePeripheral.addAttribute(rxCharacteristic);
    blePeripheral.addAttribute(txCharacteristic);
    // begin initialization
    blePeripheral.begin();
    serial.printf("BLE UART Peripheral begin!\r\n");
    /*---------------------------------------------------------------*/
}

void loop()
{
    BLECentral central = blePeripheral.central();

    if (central) {
        // central connected to peripheral
        serial.printf("Connected to central\r\n");

        while (central.connected()) {
            // central still connected to peripheral
            if (rxCharacteristic.written()) {
                unsigned char rxlen = rxCharacteristic.valueLength();
                const unsigned char *val = rxCharacteristic.value();
                serial.printf("didCharacteristicWritten, Length: %d\r\n", rxlen);
                unsigned char i = 0;
                while(i < rxlen) {
                    serial.printf("%d, ",val[i++]);
                }
                serial.printf("\r\n");
            }
            // check if serial data available
            if(serial.readable()) { // Do not spend much time on doing other things when serial available! Otherwisee, data will be lost.
                char in = serial.getc();
                if (in == CR) {
                    serial.printf("Received from terminal: %d bytes\r\n", txlen);
                    txCharacteristic.setValue((const unsigned char *)txbuf, txlen);
                    txlen = 0;
                } else {
                    txbuf[txlen] = in;
                    txlen++;
                    if (txlen >= 16) {
                        // buffer overrun
                        txlen = 0;    
                    }  
                }
            }
        }

        // central disconnected
        serial.printf("Disconnected from central\r\n");
    }
}

int main()
{
    setup();
    while(1) loop();
}

